//+------------------------------------------------------------------+
//|                                   Original name: ProfitMeter.mqh |
//|                                    Copyright (c) 2018, Marketeer |
//|                        https://www.mql5.com/en/blogs/post/719643 |
//+------------------------------------------------------------------+

// NB. In MT4 SYMBOL_TRADE_CALC_MODE works differently to MODE_MARGINCALCMODE
#ifdef __MQL4__
#define TRADE_CALC_MODE(S) ((int)MarketInfo(S, MODE_MARGINCALCMODE))
#else
#define TRADE_CALC_MODE(S) ((int)SymbolInfoInteger(S, SYMBOL_TRADE_CALC_MODE))
#endif

#ifdef __MQL4__

// MT4 does not provide this info in API, though it's filled in specifications;
// this may lead to discrepancies with built-in MARGINREQUIRED,
// when margin percentage is used internally (not 100%) and not available from MQL4
#define SymbolInfoMarginRate(A,B,C,D) (C=D=1.0)

#define SYMBOL_CALC_MODE_FOREX 0
#define SYMBOL_CALC_MODE_CFD 1
#define SYMBOL_CALC_MODE_FUTURES 2
#define SYMBOL_CALC_MODE_CFDINDEX 3
#define SYMBOL_CALC_MODE_CFDLEVERAGE 4

#define SYMBOL_CALC_MODE_FOREX_NO_LEVERAGE -1
#define SYMBOL_CALC_MODE_EXCH_STOCKS -2
#define SYMBOL_CALC_MODE_EXCH_FUTURES -3
#define SYMBOL_CALC_MODE_EXCH_FUTURES_FORTS -4

#endif

// NB. In MT5, built-in OrderCalcMargin is not allowed to work in indicators
//     In MT4, the same limitation applies to AccountFreeMarginCheck
bool MyOrderCalcMargin(const ENUM_ORDER_TYPE action, const string symbol, double volume, double price, double &margin)
{
  double MarginInit, MarginMain;
  MqlTick Tick;
  
  if((action != ORDER_TYPE_BUY && action != ORDER_TYPE_SELL) || volume < 0 || price < 0) return false;
  
  if(!SymbolInfoTick(symbol, Tick)) return false;

  if(!SymbolInfoMarginRate(symbol, action, MarginInit, MarginMain)) return false;
  
  if(volume == 0) volume = SymbolInfoDouble(symbol, SYMBOL_VOLUME_MIN);
  
  double contract = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
  long leverage = AccountInfoInteger(ACCOUNT_LEVERAGE);

  if(price == 0) price = action == ORDER_TYPE_BUY ? Tick.ask : Tick.bid;
  
  margin = 0;
  
  switch(TRADE_CALC_MODE(symbol))
  {
    case SYMBOL_CALC_MODE_FOREX_NO_LEVERAGE:
      leverage = 1;
      
    case SYMBOL_CALC_MODE_FOREX:
      margin = volume * contract / leverage * MarginInit;
      break;

    case SYMBOL_CALC_MODE_CFD:      
      margin = volume * contract * price * MarginInit;
      break;
      
    case SYMBOL_CALC_MODE_CFDINDEX:
      margin = volume * contract * price * SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE) / SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE) * MarginInit;
      break;
      
    case SYMBOL_CALC_MODE_CFDLEVERAGE:
      margin = volume * contract * price / leverage * MarginInit;
      break;
      
    case SYMBOL_CALC_MODE_EXCH_STOCKS:
      if(price == 0) price = Tick.last;
      // else if(price != Tick.last) return false;
      margin = volume * contract * price * MarginInit;
      break;

    case SYMBOL_CALC_MODE_FUTURES:
    case SYMBOL_CALC_MODE_EXCH_FUTURES:
    case SYMBOL_CALC_MODE_EXCH_FUTURES_FORTS: // case 34: // SYMBOL_CALC_MODE_EXCH_OPTIONS
      margin = volume * SymbolInfoDouble(symbol, SYMBOL_MARGIN_INITIAL) * MarginInit; 
      // margin = volume * SymbolInfoDouble(symbol, SYMBOL_MARGIN_MAINTENANCE) * MarginMain; 
      break;
  }
  
  string account = AccountInfoString(ACCOUNT_CURRENCY);
  string current = SymbolInfoString(symbol, SYMBOL_CURRENCY_MARGIN);
  if(current != account)
  {
    if(!Convert(current, account, margin)) return false;
  }
  
  return true;
}

int FindExchangeRate(const string current, const string account, string &result)
{
  for(int i = 0; i < SymbolsTotal(true); i++)
  {
    string symbol = SymbolName(i, true);
    if(TRADE_CALC_MODE(symbol) == SYMBOL_CALC_MODE_FOREX
    || TRADE_CALC_MODE(symbol) == SYMBOL_CALC_MODE_FOREX_NO_LEVERAGE)
    {
      string base = SymbolInfoString(symbol, SYMBOL_CURRENCY_BASE);
      string profit = SymbolInfoString(symbol, SYMBOL_CURRENCY_PROFIT);
      if(base == current && profit == account)
      {
        result = symbol;
        return +1;
      }
      else
      if(base == account && profit == current)
      {
        result = symbol;
        return -1;
      }
    }
  }
  return 0;
}

bool Convert(const string current, const string account, double &margin)
{
   string rate;
   int dir;
     
   if(account != "USD") {
     dir = FindExchangeRate("USD", account, rate);
     if(dir == +1) margin *= SymbolInfoDouble(rate, SYMBOL_BID);
     else
     if(dir == -1) margin /= SymbolInfoDouble(rate, SYMBOL_BID);
     else
     {
       static bool once1 = false;
       if(!once1)
       {
         Print("Can't convert USD -> ", account);
         once1 = true;
       }
       //bStop = true;
       //return false;
     }
   }
   
   if(current != "USD") {
     dir = FindExchangeRate(current, "USD", rate);
     if(dir == +1) margin *= SymbolInfoDouble(rate, SYMBOL_BID);
     else
     if(dir == -1) margin /= SymbolInfoDouble(rate, SYMBOL_BID);
     else
     {
       static bool once2 = false;
       if(!once2)
       {
         Print("Can't convert ", current, " -> USD");
         once2 = true;
       }
       //bStop = true;
       //return false;
     }
   }
   return true;
}

double PointValue(const string symbol)
{
  double point = SymbolInfoDouble(symbol, SYMBOL_POINT);
  double contract = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
  double result = 0;

  switch(TRADE_CALC_MODE(symbol))
  {
    case SYMBOL_CALC_MODE_FOREX_NO_LEVERAGE:
    case SYMBOL_CALC_MODE_FOREX:
    case SYMBOL_CALC_MODE_CFD:
    case SYMBOL_CALC_MODE_CFDINDEX:
    case SYMBOL_CALC_MODE_CFDLEVERAGE:
    case SYMBOL_CALC_MODE_EXCH_STOCKS:
      result = point * contract;
      break;

    case SYMBOL_CALC_MODE_FUTURES:
    case SYMBOL_CALC_MODE_EXCH_FUTURES:
    case SYMBOL_CALC_MODE_EXCH_FUTURES_FORTS: // case 34: // SYMBOL_CALC_MODE_EXCH_OPTIONS
      result = point * SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE) / SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
      break;
  }
  
  string account = AccountInfoString(ACCOUNT_CURRENCY);
  string current = SymbolInfoString(symbol, SYMBOL_CURRENCY_PROFIT);
  if(current != account)
  {
    if(!Convert(current, account, result)) return 0;
  }
  
  return result;
}
